#!/usr/bin/env -S node --no-warnings --loader ts-node/esm
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 *   Sidecar - https://github.com/huan/sidecar
 *
 *   @copyright 2021 Huan LI (李卓桓) <https://github.com/huan>
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */
const tstest_1 = require("tstest");
const mod_js_1 = require("../src/mod.js");
const factorial_sidecar_js_1 = require("../examples/dynamic-library/factorial-sidecar.js");
(0, tstest_1.test)('library export function call', async (t) => {
    if (process.platform !== 'linux' && process.platform !== 'win32') {
        void t.skip('This test will be skipped because it only support Linux(.so) and Windows(.dll) now');
        return;
    }
    const sidecar = new factorial_sidecar_js_1.FactorialSidecar();
    await (0, mod_js_1.attach)(sidecar);
    const EXPECTED_RET_VALUE = 6;
    const ret = await sidecar.factorial(3);
    await (0, mod_js_1.detach)(sidecar);
    t.equal(ret, EXPECTED_RET_VALUE, 'should get the factorial(3) = 6');
});
//# sourceMappingURL=library-sidecar.spec.js.map