#!/usr/bin/env -S node --no-warnings --loader ts-node/esm
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
import { test } from 'tstest';
import { Ret } from '../../ret.js';
import { SidecarBody } from '../../sidecar-body/sidecar-body.js';
import { Call } from '../call/call.js';
import { Hook } from '../hook/hook.js';
import { ParamType } from '../param-type/param-type.js';
import { RetType } from '../ret-type/ret-type.js';
import { getMetadataSidecar } from './metadata-sidecar.js';
import { Sidecar } from './sidecar.js';
const getFixture = () => {
    let Test = class Test extends SidecarBody {
        testMethod(content, n) { return Ret(content, n); }
        hookMethod(n) { return Ret(n); }
    };
    __decorate([
        Call(0x42),
        RetType('pointer', 'Utf8String'),
        __param(0, ParamType('pointer', 'Utf8String')),
        __param(1, ParamType('int')),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [String, Number]),
        __metadata("design:returntype", Promise)
    ], Test.prototype, "testMethod", null);
    __decorate([
        Hook(0x17),
        __param(0, ParamType('int')),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Number]),
        __metadata("design:returntype", void 0)
    ], Test.prototype, "hookMethod", null);
    Test = __decorate([
        Sidecar('chatbox')
    ], Test);
    return Test;
};
test('@Sidecar() smoke testing', async (t) => {
    let Test = class Test extends SidecarBody {
    };
    Test = __decorate([
        Sidecar('chatbox')
    ], Test);
    const test = new Test();
    t.equal(Test.name, 'Test', 'should have the original class name after @Sidecar decorated');
    t.ok(test, 'should instanciate decorated class successfully');
});
test('@Sidecar() viewMetadata()', async (t) => {
    const Test = getFixture();
    const metadata = getMetadataSidecar(Test);
    const EXPECTED_DATA = {
        initAgentScript: undefined,
        interceptorList: [
            {
                address: {
                    name: 'hookMethod',
                    paramTypeList: [
                        [
                            'int',
                        ],
                    ],
                    retType: undefined,
                    target: { address: '0x17', moduleName: null, type: 'address' },
                },
            },
        ],
        nativeFunctionList: [
            {
                address: {
                    name: 'testMethod',
                    paramTypeList: [
                        [
                            'pointer',
                            'Utf8String',
                        ],
                        [
                            'int',
                        ],
                    ],
                    retType: [
                        'pointer',
                        'Utf8String',
                    ],
                    target: { address: '0x42', moduleName: null, type: 'address' },
                },
            },
        ],
        sidecarTarget: {
            target: 'chatbox',
            type: 'process',
        },
    };
    t.same(metadata, EXPECTED_DATA, 'should get view from metadata correct');
});
//# sourceMappingURL=sidecar.spec.js.map